@@ -8,9 +8,8 @@ package Text::Layout;
 
 use Carp;
 
-use Text::Layout::Version;
 
-our $VERSION = $Text::Layout::Version::VERSION;
+our $VERSION = "0.012";
 
 =head1 NAME
 
@@ -31,6 +30,9 @@ description.
 
 Example, using PDF::API2 integration:
 
+    use PDF::API2;		# or PDF::Builder
+    use Text::Layout;
+
     # Create a PDF document.
     my $pdf = PDF::API2->new;	# or PDF::Builder->new
     $pdf->mediabox( 595, 842 );	# A4, PDF units
@@ -50,7 +52,8 @@ Example, using PDF::API2 integration:
     $layout->set_alignment("center");
 
     # Render it.
-    $layout->show( $x, $y, $ctx );
+    $layout->show( 0, 600, $ctx );
+    $pdf->saveas("out.pdf");
 
 All PDF::API2 graphic and text methods can still be used, they won't
 interfere with the layout methods.
@@ -140,12 +143,9 @@ sub new {
 	shift;
 	$be = shift;
     }
-    if ( $INC{"PDF/API2.pm"} ) {
+    if ( $INC{"PDF/API2.pm"} || $INC{"PDF/Builder.pm"} ) {
 	$be = __PACKAGE__."::PDFAPI2";
     }
-    elsif ( $INC{"PDF/Builder.pm"} ) {
-	$be = __PACKAGE__."::PDFBuilder";
-    }
     elsif ( $INC{"Cairo.pm"} ) {
 	$be = __PACKAGE__."::Cairo";
     }
@@ -279,6 +279,8 @@ sub get_text {
 
 Returns the number of characters in the text of this layout.
 
+Basically the same as length of get_text().
+
 Returns undef if no text has been set.
 
 =back
@@ -422,7 +424,6 @@ sub set_markup {
 		# <span rise=324>
 		elsif ( $k eq "rise" ) {
 		    $base += $v / 1024 * $fsiz;
-		    warn("base=$base\n");
 		}
 
 		# <span strikethrough=false>
